$title  A Static General Equilibrium Analysis

*	The following statements read the defines file and then 
*	deletes it:

$if exist defines.inc $include defines.inc
$if exist defines.inc $call 'del defines.inc >nul'

$libinclude cgi input policy     uniform  "Carbon tax policy" exempt rebate ver tariff
$libinclude cgi input wage       flexible "Wage rates" fixed
$libinclude cgi input homogcoal  no       "Treat coal as homogeneous" yes
$libinclude cgi input baseline   ref	  "IEO baseline" lo hi
$libinclude cgi input capital    regional "Capital mobility" sectorspecific global 
$libinclude cgi input carbtarget 7.4	  "Leakage compensated target" 8.0 no


*	Exogenous assumption: no trading in emission permits

*.$libinclude cgi input trading    none     "Carbon permit trading"  OECD AnnexB GL-CH Global 

$setglobal trading none

$set exempt no
$if %policy%==exempt $set exempt yes
$set rebate no
$if %policy%==rebate $set rebate yes
$set border no
$if %policy%==tariff $set border yes

*	Specify the dataset:

*	(Alternative choices include 13x8 25x8 25x14 25x23 45x14)

$setglobal dataset 13x8

*	Some additional switches:

$set sinkptl 0

$batinclude ieodata 

SET	time(t)	 Time period of the analysis / 2010 /,

	Trading	 Alternative trading scenarios
		 /none, OECD, AnnexB, GL-CH, Global /,

	SC	 Energy scenarios in the IEO statistics /ref, lo, hi/,

	NONAB(r) Non-AnnexB countries 

	EIT(r)   Economies in transition

	EUFSU(r) EU and FSU block

	NEUAB(r) Non-EU AnnexB countries; 

set	border(i,s,r)	Set of trade flows subject to border adjustment;
border(i,s,r) = no;

set	rebate(i,r)	Rebate flag
	exempt(i,r)	Exemption flag;

rebate(i,r) = no;
exempt(i,r) = no;

*	Define the various regions:

EIT(r)   = yes$(annexb(r) AND not oecd(r)) ;
NEUAB(r) = yes$(annexb(r) AND not EU(r)) ;
EUFSU(r) = yes$(EIT(r) OR EU(r)) ;
NONAB(r) = yes$(not Annexb(r));
DISPLAY NONAB,NEUAB, EIT,EUFSU;


SCALAR	CARBTAX	Flag for a carbon tax in the model /0/
	SSK	Flag for sector-specific capital /0/,
	RSK	Flag for region-specific capital /1/,
	GK	Flag for global capital		 /0/
	CARBTARGET  Flag for leakage-compensated limits /0/,
	PINDEX	Price index for normalization;

$if %capital%==sectorspecific	SSK = 1;
$if not %capital%==regional	RSK = 0;
$if %capital%==global		GK = 1;

PARAMETER	MaxSinkPtl(r) Maximum sink potential ;
maxsinkptl(r) = 0;

* as specified by EMF Feb 27 1998 in billions of tonnes 

*	   / USA 0.300, EUR 0.150, OOE 0.150,FSU 0.600,MPC 0.300, ROW 0.900/ ;
	   
*	Additional parameters required for the static model:

PARAMETER
	ks(r)		Capital stock
	id(r)		Investment demand
	xtax		Export tax flag

	trdscn(trading)	Flag for current trading scenario

	fe0		Factor endowments value
	carbon90	1990 regional carbon emissions
	baucompare      comparison of DOE and model emissions,
	ratioemit	Ratio of emissions (IEO vs BaU),
	bauemit		BaU global carbon emissions
	ptarget		Target price
	eprd(i,r)	Energy production (Exajoules)

	pnum		Numeraire price index for presentvalue GDP
	pcnum		Numeraire price index for future GDP

	bauw(r)		Welfare index in baseline
	baucpi		BaU consumer price index
	bauconsum	Present value consumption,
	baucapital	Present value rental income on capital,
	bauprref	Relative return to fixed energy resource in BAU

	baugdp		Baseline GDP
	bauenc		Aggregate energy consumption (ej)
	baueleenc	Energy consumption by Electric Utilities(ej)
	baucarbon	Total carbon emissions along baseline,
	totbaucrb	Global carbon emissions along baseline,

	bauimport	Baseline trade flows 
	bauwage		BaU real wage
	baureturn	BaU return to capital
	baurent		BaU resource rent
	baupp		BaU producer price
	baucp		BaU consumer price (gross of carbon tax)
	bautrade	BaU homogeneous energy trade
	bauprc		BAU present value price of energy at well head (10 dollars per GJ or MWh)
	baup_cru	Basline crude price (dollar per barrel)
	bauc		BAU consumption
	bauy		BAU gross output by industry
	bauttly		BAU total output 
	bauk		BAU sectoral capital

	scnw(r)		Welfare index through 2030
	scncpi		Scenario consumer price index
	scnconsum	Present value consumption,

	scngdp		Baseline GDP
	scnenc		Aggregate energy consumption (ej)
	scneleenc	Energy consumption by Electric Utilities(ej)
	scncarbon	Total carbon emissions along baseline,
	scncapital	Present value rental income on capital,
	scnimport	Baseline trade flows 
	scnterms	Terms of trade (Laspeyres export and import price indexes)
	scnwage		SCN real wage
	scnreturn	SCN return to capital
	scnrent		SCN resource rent
	scnpp		SCN producer price
	scncp		SCN consumer price (gross of carbon tax)
	scntrade	Scenario homogeneous energy trade
	scnprc(*,t,r)	SCN present value price of energy at well head (10 dollars per GJ or MWh)
	scnp_cru	Basline crude price (dollar per barrel)
	scnc(t,r)	scn consumption
	scny(i,t,r)	scn gross output by industry
	scnttly(t,r)    SCN total output 
	scnk(*,t,r)	scn sectoral capital

	cpct		Consumption (%)
	ur		Unemployment (%)
	gdppct		GDP (%)
	ctaxrate	Carbon tax 
	ctaxpct		Percentage carbon tax

	ctax(t,*)	Carbon tax rates by region and world market
	leakage(t,*)	Leakage rates: total and by region
	deltagdp(*,*)	Change in GDP
	pctgdp(*,*)	Percentage change in GDP
	pctprc(whe,*,r)	Pct future value price of energy at well head (%)

	carbpct(t,r)	Carbon permit imports as percent of cutback
	carbval(t,*)	Value of net carbon permit imports (billions of dollars)
	carbimp(t,*)	Carbon permit imports 
	carbdom(t,*)    Percentage of obligation met domestically


	carbon(r,i)	Carbon contents uses for border adjustment
	harbtg(t,*)     Harberger triangle
	dwl(t,*)	Deadweight loss from carbon tax

	pctcpi(t,*)	Percentage change in price index
	pctterms(t,*)	Terms of trade (%)

	terms		Change in terms of trade 
	prices		Percentage change in producer prices
	cprices		Percentage change in consumer prices
	energytrd	Energy change in net exports
	output		Percentage change in output for 2010
	export		Percentage change in exports
	import		Percentage change in imports
	totleakage	Leakage 
	regleakage	Regional leakage

	verpct		VER revenue as percentage of welfare

	vem(wmkt,r)	Energy import value coefficient
	emc(wmkt,r)	Energy import coefficient
	et(wmkt,r)	Energy transport cost coefficient


	tcarb(i,*,r)	Ad-valorem tax rates for implementing carbon constraints

	abatement	Level of abatement

	ver(i,r,s)	Voluntary export restraint;

ver(i,r,s) = 0;

scalar unempl Swtich for unemployment /0/;

parameter	plmin(r)	Minimum wage (for unemployment model);

plmin(r) = 0;

tcarb(i,j,r) = 0;

*	Assign import values and transport costs:

loop((wmkt,r),
  if (em0(wmkt,r),
	vem(wmkt,r) = vem0(wmkt,r);
	emc(wmkt,r) = em0(wmkt,r);
	et(wmkt,r) = et0(wmkt,r);
  else
	vem(wmkt,r) = 1;
	emc(wmkt,r) = 1;
	et(wmkt,r) = 0.1
  );
);


*----------------------------------------------------------------------------------------
*	First define carbon contents for energy goods in physical units
*	-- carbon per exajoule or TKwH:

eprd(i,r)     = sum(j, eind(i,j,r)) + efd(i,r) - eimp(i,r) + eexp(i,r) ;

carbon(r,nele) = cecphys(nele);
carbon(r,ele) = sum(nele, cecphys(nele) * eind(nele,ele,r))/
			(sum(i, eind(ele,i,r)) + efd(ele,r));

*	Then use these carbon contents to define the carbon content of
*	goods in grams per dollar:

loop((i,r)$vom(i,r),
  carbon(r,i)$(not e(i)) = sum(e, carbon(r,e)*eind(e,i,r))/vom(i,r);
);

*	Finally, restate the carbon content of energy goods in the
*	same units (grams per dollar):

carbon(r,e)$vom(e,r) = carbon(r,e) * eprd(e,r) / vom(e,r);
carbon(r,e)$(not vom(e,r)) = 0;

display carbon;

carbcoef(fe,i,r)       = cecphys(fe) * eind(fe,i,r);
carbcoef(fe,"final",r) = cecphys(fe) * efd(fe,r);

$ontext

$model:gtap

$sectors:
        c(r)			! Private consumption
        y(i,r)$vom(i,r)		! Output
	x(i,r)$vxm(i,r)
        a(i,r)$a0(i,r)		! Armington aggregation of domestic and imports

	emit(r)			! Carbon emissions

        em(wmkt,r)$(em0(wmkt,r) or ymkt(wmkt,r))	! Energy imports
        ex(wmkt,r)$ex0(wmkt,r)	! Energy exports

	em_a(wmkt)$(sum(r$(rtax(r) or ttax(r)), em0(wmkt,r)))
	ex_a(wmkt)$(sum(r$(rtax(r) or ttax(r)), ex0(wmkt,r)))

	carbonm(r)$ttax(r)	! Carbon rights imports
	carbonx(r)$ttax(r)	! Carbon rights exports
	cemit(i,r)$(rebate(i,r) or exempt(i,r))	! Carbon allocation for rebate sector

	yt			! Transport 
	backstop(r)$carbtax	! Carbon backstop

$commodities:
        pc(r)                   ! Final demand
        pe(i)$wmkt(i)           ! International market energy price
        pe_a(i)$(wmkt(i) and (card(rtax) or card(ttax)))
        py(i,r)$ymkt(i,r)       ! Output price price
	px(i,r)$vxm(i,r)
        pa(i,r)$a0(i,r)         ! Armington composite price
        pl(r)                   ! Wage rate
        pr(i,r)$rd0(i,r)        ! Energy resource
        rk(i,r)$(ssk*kd0(i,r))	! Return to sector-specific capital
	rkr(r)$rsk		! Return to regional capital
	rkg$gk			! Return to global capital
        pt                      ! Transport services
	pq(i,s,r)$ver(i,s,r)	! VER quota rent

	pcarbon(r)					! Carbon price
        pcarb(r)$(rtax(r) OR ttax(r))			! Carbon permit price
	pcemit(i,r)$(rebate(i,r) or exempt(i,r))	! Carbon permit price -- rebate

        pwcarb$card(ttax)	! World carbon permits

$consumers:
        ra(r)                   ! Representative agent

$auxiliary:
	u(r)$unempl		! Unemployment rate 
	target$carbtarget
	tau(i,s,r)$border(i,s,r) ! Border taxes on carbon content
        rs(xe,r)$rd0(xe,r)      ! Resource supply index
        rslvl(i)$wmkt(i)        ! Resource supply for international resources
	sx(i,r)$rebate(i,r)	! Rebate subsity

$prod:emit(r)
	o:pcarbon(r)			q:1
	i:pcarb(r)$(rtax(r) or ttax(r))	q:1
	i:pc(r)				q:1.e-5

*       Final demand

$prod:c(r)  s:(esub("final",t,r)) c:1 e:1 nele(e):(isub(t,r)) oil(e):0 col(e):0 gas(e):0
    o:pc(r)		q:ct0(r)
    i:pa(i,r)	q:(aeei(i,t,r)*c0(i,r))   p:pc0(i,r) i.tl:$fe(i) c:$(not e(i)) e:$ele(i)
+                       a:ra(r)		  t:tc(i,r)   t:tcarb(i,"final",r)
    i:pcarbon(r)#(fe)   q:(aeei(fe,t,r)*carbcoef(fe,"final",r)) p:1e-5  fe.tl:

$report:
        v:edf(i,r)$(e(i) and c0(i,r)) i:pa(i,r) prod:c(r)	! energy in final demand
	v:fd(xe,r)                    o:py(xe,r) prod:y(xe,r)   ! fossil fuel supply

*       Non-fossil fuel production (excluding electricity):

$prod:y(i,r)$(nr(i,r) and not ele(i))   s:0  vae(s):(esub(i,t,r))  va(vae):e_vae(r) 
+                       e(vae):(e_elec(t,r))  nel(e):(e_c_go(i,t,r)) lqd(nel):(e_g_o(i,t,r))
+                       oil(lqd):0 col(nel):0 gas(lqd):0 cru(s):0 

	o:py(i,r)		q:vom(i,r)  a:ra(r) t:ty(i,r) 

	i:pa(j,r)$(not fe(j))	q:vafm(j,i,r) p:pai0(j,i,r) e:$ele(j) a:ra(r) t:ti(j,i,r)
        i:pl(r)			q:ld0(i,r)                      va:

        i:rk(i,r)$ssk		q:kd0(i,r)                      va:
        i:rkr(r)$rsk		q:kd0(i,r)                      va:
        i:rkg$gk		q:kd0(i,r)                      va:
	i:pr(i,r)		q:rd0(i,r)			va:
        i:pcarbon(r)#(fe)$(not rebate(i,r))	q:(aeei(fe,t,r)*carbcoef(fe,i,r)) p:1e-5 fe.tl:
        i:pcemit(i,r)#(fe)$rebate(i,r)		q:(aeei(fe,t,r)*carbcoef(fe,i,r)) p:1e-5 fe.tl:
        i:pa(fe,r)	q:(aeei(fe,t,r)*vafm(fe,i,r)) p:(pfe0(fe,i,t,r))	fe.tl: 
+		a:ra(r) t:ti(fe,i,r)   t:tcarb(fe,i,r)

$prod:y(i,r)$ele(i)   s:0  vae(s):1  va(vae):e_vae(r) fuel:4 gas(fuel):0 col(fuel):0
+			oil(s):0 cru(s):0
	o:py(i,r)		q:vom(i,r)  a:ra(r) t:ty(i,r) 

	i:pa(j,r)$(not fe(j))	q:vafm(j,i,r) p:pai0(j,i,r) e:$ele(j) a:ra(r) t:ti(j,i,r)
        i:pl(r)			q:ld0(i,r)                      va:

        i:rk(i,r)$ssk		q:kd0(i,r)                      va:
        i:rkr(r)$rsk		q:kd0(i,r)                      va:
        i:rkg$gk		q:kd0(i,r)                      va:
	i:pr(i,r)		q:rd0(i,r)			va:
        i:pcarbon(r)#(fe)	q:(aeei(fe,t,r)*carbcoef(fe,i,r)) p:1e-5   fe.tl:
        i:pa(fe,r)		q:(aeei(fe,t,r)*vafm(fe,i,r)) p:(pfe0(fe,i,t,r)) fe.tl: 
+		a:ra(r) t:ti(fe,i,r)   t:tcarb(fe,i,r)


$report:
        v:ed(j,i,r)$(fe(j) and vafm(j,i,r)) i:pa(j,r) prod:y(i,r)

*       Fossil fuel production activity (crude, gas and coal):

$prod:y(xe,r)$vom(xe,r)  s:(esub_es(xe,t,r))  id:0 

        o:py(xe,r)		q:vom(xe,r)             a:ra(r) t:ty(xe,r) 
        i:pa(j,r)		q:vafm(j,xe,r) p:pai0(j,xe,r)  a:ra(r) t:ti(j,xe,r) id: t:tcarb(j,xe,r)
        i:pcarbon(r)#(fe)	q:carbcoef(fe,xe,r) p:1e-5 id: 
        i:pl(r)			q:ld0(xe,r)	id:
        i:rk(xe,r)$ssk		q:kd0(xe,r)	id:
        i:rkr(r)$rsk		q:kd0(xe,r)	id:
        i:rkg$gk		q:kd0(xe,r)	id:
        i:pr(xe,r)		q:rd0(xe,r)	p:(prref(xe,t,r))

*	Export supply activity:

$prod:x(i,r)$(vxm(i,r) and not exempt(i,r))
	o:px(i,r)	q:vxm(i,r)    a:ra(r) n:sx(i,r)$rebate(i,r)  m:-1$rebate(i,r)
	i:py(i,r)	q:vxm(i,r)


*	Under exemptions we permit production for export to have
*	different structure than production for domestic sales:

$prod:x(i,r)$(vxm(i,r) and exempt(i,r))   s:0  vae(s):(esub(i,t,r))  va(vae):e_vae(r) 
+                       e(vae):(e_elec(t,r))  nel(e):(e_c_go(i,t,r)) lqd(nel):(e_g_o(i,t,r))
+                       oil(lqd):0 col(nel):0 gas(lqd):0 cru(s):0 
	o:px(i,r)		q:vom(i,r)  a:ra(r) t:ty(i,r) 
	i:pa(j,r)$(not fe(j))	q:vafm(j,i,r) p:pai0(j,i,r) e:$ele(j) a:ra(r) t:ti(j,i,r)
        i:pl(r)			q:ld0(i,r)                      va:
        i:rk(i,r)$ssk		q:kd0(i,r)                      va:
        i:rkr(r)$rsk		q:kd0(i,r)                      va:
        i:rkg$gk		q:kd0(i,r)                      va:
	i:pr(i,r)		q:rd0(i,r)			va:
        i:pcemit(i,r)#(fe)	q:(aeei(fe,t,r)*carbcoef(fe,i,r)) p:1e-5   fe.tl:
        i:pa(fe,r)		q:(aeei(fe,t,r)*vafm(fe,i,r)) p:(pfe0(fe,i,t,r))	fe.tl: 
+		a:ra(r) t:ti(fe,i,r) 

$prod:cemit(i,r)$exempt(i,r)
	o:pcemit(i,r)		q:1
	i:pcarbon(r)		q:1  a:ra(r)  t:(-0.999)

$prod:cemit(i,r)$rebate(i,r)
	o:pcemit(i,r)		q:1
	i:pcarbon(r)		q:1

$constraint:sx(i,r)$rebate(i,r)
	sx(i,r) * px(i,r) * vom(i,r) * y(i,r) =e= pcemit(i,r) * cemit(i,r);

*       Armington aggregation over domestic versus imports:

$prod:a(i,r)$a0(i,r)  s:esubdm(i,r)  m:esubmm(i,r) s.tl(m):0
        o:pa(i,r)       q:a0(i,r)
        i:py(i,r)       q:(dd(i,t,r)*d0(i,r))
        i:px(i,s)       q:(md(i,t,s,r)*vxmd(i,s,r))  p:pmx0(i,s,r) s.tl:
+               a:ra(s) t:tx(i,s,r)     a:ra(r) t:(tm(i,s,r)*(1+tx(i,s,r)))
+		a:ra(r) n:tau(i,s,r)$border(i,s,r)
        i:pt#(s)       q:(md(i,t,s,r)*vtwr(i,s,r))  p:pmt0(i,s,r) s.tl:
+		a:ra(r) t:tm(i,s,r)
	i:pq(i,s,r)$ver(i,s,r)	q:(md(i,t,s,r)*vxmd(i,s,r))  s.tl:

$report:
	v:imports(i,s,r)$(a0(i,r)*vxmd(i,s,r))	i:px(i,s)	prod:a(i,r)

$prod:em(wmkt,r)$ymkt(wmkt,r)
        o:py(wmkt,r)				q:vem(wmkt,r)
        i:pe(wmkt)$(not (rtax(r) or ttax(r)))	q:emc(wmkt,r) a:ra(r) t:tme(wmkt,r)
        i:pe_a(wmkt)$(rtax(r) or ttax(r))	q:emc(wmkt,r) a:ra(r) t:tme(wmkt,r)
        i:pt				        q:et(wmkt,r) a:ra(r) t:tme(wmkt,r)

$prod:ex(wmkt,r)$ex0(wmkt,r)
        o:pe(wmkt)$(not (rtax(r) or ttax(r)))	q:ex0(wmkt,r)
	o:pe_a(wmkt)$(rtax(r) or ttax(r))	q:ex0(wmkt,r)
        i:py(wmkt,r)				q:vex0(wmkt,r)  a:ra(r) t:txe(wmkt,r)  

$prod:ex_a(wmkt)$(sum(r$(rtax(r) or ttax(r)), ex0(wmkt,r)))
	o:pe(wmkt)		q:1
	i:pe_a(wmkt)		q:1

$prod:em_a(wmkt)$(sum(r$(rtax(r) or ttax(r)), em0(wmkt,r)))
	o:pe_a(wmkt)	q:1
	i:pe(wmkt)	q:1.001

$constraint:tau(i,s,r)$border(i,s,r)
        py(i,s)*tau(i,s,r) =e= pcarb(r)*carbon(s,i);

* Carbon Activities

$prod:carbonm(r)$ttax(r)
	o:pcarb(r)	q:1
	i:pwcarb	q:1

$prod:carbonx(r)$ttax(r)
	o:pwcarb	q:1
	i:pcarb(r)	q:1     a:ra(r)   t:(mkup(t,r))

*       International transport services (Cobb-Douglas):

$prod:yt  s:1
        o:pt           q:(sum((i,r), vst(i,r)))
        i:py(i,r)       q:vst(i,r)

$prod:backstop(r)$carbtax
	o:pcarb(r)$(rtax(r) OR ttax(r))	q:1
	i:pl(r)				q:(backprc(t,r))

*       Final demand:

$demand:ra(r)   
        d:pc(r)		q:(ct0(r)*gdp(t,r))	
	e:pq(i,r,s)	q:ver(i,r,s)
	e:py(cgd,r)	q:(-id(r))
	e:rk(i,r)$ssk	q:(thetak(i,r)*ks(r))
	e:rkr(r)$rsk	q:ks(r)
	e:rkg$gk	q:ks(r)
        e:py(wmkt,r)	q:(mktadj(wmkt,r)*a0(wmkt,r)*gdp(t,r))
        e:pl(r)		q:(evoa("lab",r)*gdp(t,r))
	e:pl(r)$unempl	q:(-evoa("lab",r)*gdp(t,r))  r:u(r)
        e:pr(xe,r)	q:(rd0(xe,r)*ffsupply(t,xe,r))  r:rs(xe,r)
        e:pr(i,r)$(not xe(i))	q:(rd0(i,r)*gdp(t,r))
        e:pc("usa")	q:(vb(r)*gdp(t,r))
        e:pcarb(r)$(rtax(r) or ttax(r))   q:(carblim(r,t))	r:target$carbtarget

$constraint:u(r)$unempl
	pl(r) =g= pc(r) * plmin(r);

$constraint:target$carbtarget
	sum(r, emit(r)) =e= carbtarget;

$report:
	v:w(r)	w:ra(r)

*       In the BAU calculation we adjust resource endowments of coal
*       and natural gas to calibrate the model to given target price:

$constraint:rs(xe,r)$rd0(xe,r)
        + ( ptarget(xe,r) * pc(r) - py(xe,r) )$(not wmkt(xe))
        + ( rs(xe,r) - rslvl(xe)             )$wmkt(xe)  =e= 0;

*       Multiplicative scaling of resources achives a given world price:

$constraint:rslvl(wmkt)
       ptarget(wmkt,"usa") * pc("usa") =e= pe(wmkt);

$offtext
$sysinclude mpsgeset gtap
gtap.workspace = 15;
gtap.reslim    = 8000;

*	======================================================================
*	Install the balanced growth path:

rs.l(xe,r)$rd0(xe,r) = 1;
rslvl.l(wmkt) = 1;

*	Reference growth path:

qref(t) = (1+g0)**(year(t)-1995);
pref(t) = (1/(1+r0))**(year(t)-1995);
pref(t) = pref(t) / sum(tfirst, pref(tfirst));

alias (r,rr);

loop(time(t),


	c.l(r) = qref(t);
	y.l(i,r) = qref(t);
	a.l(i,r) = qref(t);
	x.l(i,r) = qref(t);

	yt.l = qref(t);

	em.l(wmkt,r) = qref(t)$em0(wmkt,r);
	ex.l(wmkt,r) = qref(t);
	em_a.l(wmkt) = qref(t);
	ex_a.l(wmkt) = qref(t);

	ex.l(wmkt,r)$(not ex0(wmkt,r)) = 0;
	em.l(wmkt,r)$(not em0(wmkt,r)) = 0;

*	Install steady-state equilibrium values:

	pcarb.l(r) = 0;
	pwcarb.l   = 0;

*	For purposes of scaling the transfer price to unity, assign
*	a present value of tax revenue:

*	Omit carbon from our check of the balanced growth path:

	carbtax = 0;

	carblim(r,t) = 0;
	mkup(t,r) = 0 ;

	ks(r) = rk0 * k0(r) * qref(t);
	id(r) = sum(cgd, vom(cgd,r)) * qref(t);
	ptarget(i,r) = 1;

	gdp(t,r)  = qref(t);

	ffsupply(t,xe,r) = qref(t)$rd0(xe,r);

	md(i,t,s,r)$a0(i,r) = gdp(t,s) * a0(i,r) /
		( gdp(t,r)*d0(i,r) + sum(rr, gdp(t,rr) * 
		(vxmd(i,rr,r)*pmx0(i,rr,r)+ vtwr(i,rr,r)*pmt0(i,rr,r))) );
	md(wmkt,t,s,r) = 0;
	dd(i,t,r)$a0(i,r) = gdp(t,r) * a0(i,r) /
		( gdp(t,r)*d0(i,r) + sum(rr, gdp(t,rr) * 
		(vxmd(i,rr,r)*pmx0(i,rr,r)+ vtwr(i,rr,r)*pmt0(i,rr,r))) );
	dd(wmkt,t,r) = 1;


	vxm(i,r) = sum(s, md(i,t,r,s)*vxmd(i,r,s));


*       AEEI equals unity to replicate the balanced growth path:

	aeei(i,tt,r) = 1;

*	No quotas apply here:

	xt(wmkt,t,r) = 0;
	xtax(wmkt,r) = 0;

*	No carbon tax in this calculation:
 
	carbtax = 0;

*	No backstop prices are loaded yet:

	backstop.l(r) = 0;
	backprc(t,r) = 0;

	pcarbon.l(r) = 1.e-5;
	emit.l(r) = qref(t) * sum(fe,cecphys(fe) * (sum(i,eind(fe,i,r)) + efd(fe,r)));

*       Replicate the 1995 calibrated benchmark -- the deviation should be 
*       on the order of 1e-5:

	gtap.iterlim = 0;
$include gtap.gen
	solve gtap using mcp;
	display "The following reports precision of the benchmark dataset:", gtap.objval;
);

*	After checking the balanced growth path, 
*	recalibrate to IEO growth path:

uniformg = 0;
$include recalib

*	======================================================================
*       Install assumptions for the projected benchmark:

*	Use DOE reference estimates for crude oil, refined oil
*	and natural gas prices:

pytarget(cru,t,r) = IEOPRICE("%baseline%",t);
pytarget(oil,t,r) = IEOPRICE("%baseline%",t);
pytarget(gas,t,r) = IEOPRICE("%baseline%",t);

*	Assume no change in coal prices under BaU:

pytarget(coal,t,r) = 1;

*	Assign the energy price targets:

pfe0(fe,i,t,r)$pytarget(fe,t,r) = pfe0(fe,i,t,r) * pytarget(fe,t,r);

*	Use the GDP growth path which is consistent with IEO statistics:

gdp(tt,r) = ieogdp("%baseline%",tt,r);

md(i,t,s,r)$a0(i,r) = gdp(t,s) * a0(i,r) /
	( gdp(t,r)*d0(i,r) + sum(rr, gdp(t,rr) * 
		(vxmd(i,rr,r)*pmx0(i,rr,r)+ vtwr(i,rr,r)*pmt0(i,rr,r))) );
md(wmkt,t,s,r) = 0;
display md;

dd(i,t,r)$a0(i,r) = gdp(t,r) * a0(i,r) /
	( gdp(t,r)*d0(i,r) + sum(rr, gdp(t,rr) * 
		(vxmd(i,rr,r)*pmx0(i,rr,r)+ vtwr(i,rr,r)*pmt0(i,rr,r))) );
dd(wmkt,t,r) = 1;
display dd;

display gdp;

*       Generate AEEI coefficients to approximate the DOE carbon emission 
*       projections:

aeei(fe,t,r) = ieocarbon("%baseline%",fe,r,t) / gdp(t,r);

aeeirate(fe,tt+1,r)$aeei(fe,tt+1,r) = 100 * ( (aeei(fe,tt,r)/aeei(fe,tt+1,r))**(1/5) - 1)
display aeeirate;

growthrate(r,tt+1,"carbon") = 100 * ( 
	(sum(fe,(carbon95(r,fe)*ieocarbon("%baseline%",fe,r,tt+1)))/
	 sum(fe,(carbon95(r,fe)*ieocarbon("%baseline%",fe,r,tt))))**(1/5) - 1);

growthrate(r,tt+1,"energy") = 100 * ( 
	(sum(fe,carbon95(r,fe)*ieocarbon("%baseline%",fe,r,tt+1)/cecphys(fe))/
	 sum(fe,carbon95(r,fe)*ieocarbon("%baseline%",fe,r,tt)  /cecphys(fe)) )**(1/5) - 1);

growthrate(r,tt+1,"gdp")    = 100 * ((gdp(tt+1,r)/gdp(tt,r))**(1/5) - 1);

growthrate(r,t,  "aeei")   = growthrate(r,t,"gdp") - growthrate(r,t,"energy");

option growthrate:1:1:1;
display growthrate;

*	Read the IEO projection for crude oil production -- this enters
*	the model as a regional share of global crude oil production:

ffsupply(t,cru,r) = ieocrude("%baseline%",r,t)$rd0(cru,r);

*       Assume that gas and coal production grow with baseline energy demand:

ffsupply(t,gas,r)$rd0(gas,r)  = ieocarbon("%baseline%",gas,r,t);
ffsupply(t,coal,r)$rd0(coal,r)= ieocarbon("%baseline%",coal,r,t);

*       Set capital flows and energy market adjustments 
*       to zero for the baseline growth path:

ffs(t,xe) = sum(r, vom(xe,r)*ffsupply(t,xe,r)) /  sum(r, vom(xe,r)*qref(t));

*	Set capital flows to zero:

vb(r) = 0;

*	Initial solution pass:

loop(time(t),

	ks(r) = rk0 * k0(r) * qref(t);
	id(r) = sum(cgd, vom(cgd,r)) * qref(t);
	ptarget(i,r) = pytarget(i,t,r);

	gtap.iterlim = 8000;
$include gtap.gen
	solve gtap using mcp;
);

set itadj /it1*it3/;
loop((itadj,time(t)),

	baucompare(fe,t,r,"model") = cecphys(fe) * (
	 (  (edf.l(fe,r)/c0(fe,r)) *  efd(fe,r))$c0(fe,r) +
           sum(i$vafm(fe,i,r), (ed.l(fe,i,r)/vafm(fe,i,r)) * eind(fe,i,r)) );

	baucompare(fe,t,r,"ieo") = 
	  ieocarbon("%baseline%",fe,r,t) * cecphys(fe) * (efd(fe,r)+sum(i,eind(fe,i,r)));

	baucompare(fe,t,r,"ratio")$baucompare(fe,t,r,"ieo") = 
	baucompare(fe,t,r,"model") / baucompare(fe,t,r,"ieo");

	bauemit(t,"ieo") = sum((fe,r), baucompare(fe,t,r,"ieo"));
	bauemit(t,itadj) = sum((fe,r), baucompare(fe,t,r,"model"));

	bauemit("1995","ieo") = sum(r, carbon95(r,"total"));
	bauemit("1995",itadj) = sum(r, carbon95(r,"total"));

	ratioemit(t,itadj)$bauemit(t,"ieo") = bauemit(t,itadj) / bauemit(t,"ieo");
	ratioemit("1995",itadj) = 1;

*	Adjust aeei to hit the target iteratively:

	aeei(fe,t,r)$( baucompare(fe,t,r,"ratio"))
		= aeei(fe,t,r) / baucompare(fe,t,r,"ratio");

$include gtap.gen
	solve gtap using mcp;

);

*	------------------- Generate Reports for BaU

option baucompare:3:1:1;
display baucompare, aeei, bauemit, ratioemit;

loop(time(t),
$batinclude statrep bau
);

display pnum, pcnum, bauw, baucpi, bauconsum, baucapital, 
	baugdp, bauenc, baueleenc, baucarbon, bauimport, 
	bauwage, bauprc, baup_cru, bauc, 
	bauy, bauttly, bauk;


*----------------------------------------------------------------------------------------
*	Fix resource supplies for the subsequent scenarios:

rs.fx(xe,r) = rs.l(xe,r);
rslvl.fx(i) = rslvl.l(i);

*	Compute the value share of the specfic-resource along the baseline path:

loop(time(t),

	thetar(xe,t,r)$(vom(xe,r)) = pr.l(xe,r)*rd0(xe,r) /
		((1-ty(xe,r))*py.l(xe,r)*vom(xe,r));

*	Compute the elasticity of substiution which is then consistent with
*	the specified elasticity of fossil fuel supply:

	esub_es(xe,t,r)$(1-thetar(xe,t,r)) = thetar(xe,t,r) * eta(r,xe) / (1-thetar(xe,t,r));

*	Finally, specify the reference price of the resource input.  This one
*	is a bit tricky because we want to specify the relative price of
*	resource in terms of all inputs:

	prref(xe,t,r)$vom(xe,r) = 
		(pr.l(xe,r)*rd0(xe,r) / 
		(py.l(xe,r)*vom(xe,r)*(1-ty(xe,r))-pr.l(xe,r)*rd0(xe,r)) )
		/ ( rd0(xe,r) / (vom(xe,r)*(1-ty(xe,r))-rd0(xe,r)) );

);

display thetar,esub_es,prref;

*	Check that the calibration is precise:

gtap.iterlim = 0;

loop(time(t),
$include gtap.gen
	solve gtap using mcp;
);

display "The following reports precision of the baseline:", gtap.objval;
gtap.iterlim = 8000;



*----------------------------
*	Scenario processing begins here:

*	Alternative trading scenarios:

trdscn("%trading%")   = 1;

*	--------------------------------------------------------

carbon90(r) = sum(fe, cecphys(fe) * (efd(fe,r)+sum(i,eind(fe,i,r))) * 
		   ieocarbon("%baseline%",fe,r,"1990"));
carblim(r,t) = 5 * carbon90(r) * qref(t);

display carbon90;

*	--------------------------------------------------------
*	Apply the Kyoto target as an upper bound on carbon emissions after taking
*	account of target adjustment and sink potential:

carblim(r,t)$(annexb(r) and (year(t) ge 2010)) 
	= (kyoto(r)/100) * carbon90(r) + %sinkptl% * MaxSinkPtl(r);

display carblim;


if (%carbtarget%, 
	target.l = 1;
	carbtarget = %carbtarget%;		    
);

carbtax = 1;

*	-------------------------------------------------------
*	Backstop cost:

backprc(t,r) = bp(t)/10;


*	Assign some rebates:

if (%exempt%,
  exempt("eis",annexb)$(nr("eis",annexb)) = yes;
  loop(time(t),
	exempt(i,r)$(not sum(fe, aeei(fe,t,r)*carbcoef(fe,i,r))) = no;
  );
);
if (%rebate%,
  rebate("eis",annexb)$(nr("eis",annexb)) = yes;
  loop(time(t),
	rebate(i,r)$(not sum(fe, aeei(fe,t,r)*carbcoef(fe,i,r))) = no;
  );
);
display exempt, rebate;

*	-------------------------------------------------------
*	Design flags for this trading scenario:

rtax(r) = no;
ttax(r) = no;

*	If global trading, set carbon limits for non AnnexB countries
*	using BaU values:

if (Trdscn("none"),
	rtax(r)$annexb(r) = yes;

);

if (Trdscn("OECD"),
	rtax(r)$(annexb(r) and Not oecd(r)) = yes;
	ttax(r)$oecd(r) = yes;

);

IF (Trdscn("AnnexB"),
	ttax(r)$annexb(r) = yes;
);	 

IF (Trdscn("GL-CH"),
	carblim(nonab,t)$(year(t) ge 2010)= baucarbon(t,nonab);
	ttax(r) = yes;
	carblim("chn",t) = 0;
	ttax("chn") = no;
);


IF (Trdscn("Global"),

	carblim(nonab,t)$(year(t) ge 2010)= baucarbon(t,nonab);
	ttax(r) = yes;
);

Display ttax, rtax, carblim;

loop(time(t),
	carbonx.fx(r)$(year(t) LT 2010) = 0 ;
	carbonm.fx(r)$(year(t) LT 2010) = 0 ;
);

*	-------------------------------------------------------
*	Apply border adjustments:

	if (%border%,
	  border(i,s,r)$((not (rtax(s) or ttax(s))) and 
			      (rtax(r) or ttax(r))  and
			      (vxmd(i,s,r))) = yes;
	);
	display border, carbon;

*	Labor market:

$if %wage%==fixed unempl = 1;
	if (unempl, plmin(r) = pl.l(r) / pc.l(r););

*	Introduce the VER quota at the baseline level:

$if %policy%==ver ver("eis",nonab,annexb)$(a0("eis",nonab)*vxmd("eis",nonab,annexb)) = imports.l("eis",nonab,annexb);

*	-------------------------------------------------------
*	Solve the scenario:

loop(time(t),

$include gtap.gen
	solve gtap using mcp;


*	-------------------------------------------------------
*	Extract the scenario solution values:

$batinclude statrep scn

);

*	-------------------------------------------------------
*	Generate reports of percentage changes:

cpct(r) = 100 * (scnw(r)/bauw(r) - 1);
option cpct:4;
display cpct;

ur(r) = na$(not unempl) + 100 * u.l(r)$unempl;

loop(time(t),

	gdppct(r) = 100 * (scngdp(t,r)/baugdp(t,r) - 1);
	ctax(t,r) = 10 * (pcarb.l(r)$(rtax(r) or ttax(r))) / pc.l("usa");
	ctax(t,"world") = 10* (pwcarb.l) / pc.l("usa");
	ctaxpct(r,fe)$(pa.l(fe,r)*(c0(fe,r)*(1+tc(fe,r))
			+sum(i,vafm(fe,i,r)*(1 + ti(fe,i,r)))))
		= 100 * pcarb.l(r)
		* (carbcoef(fe,"final",r) + sum(i, carbcoef(fe,i,r)))
		/ (pa.l(fe,r)* ( c0(fe,r)*(1+tc(fe,r)) + 
			sum(i,vafm(fe,i,r)*(1 + ti(fe,i,r)))));
			  
	option ctax:0, ctaxpct:0;
	display ctax, ctaxpct;

* new capital flow accounting

	Display backstop.l ;

	Display baucarbon, scncarbon;

	abatement = round(sum(rr$(rtax(rr) or ttax(rr)), baucarbon(t,rr) - carblim(rr,t)), 5);
	display abatement;
	if (abatement,
	  loop(r$(not (rtax(r) or ttax(r))),
	    leakage(t,r)
		= 100 * (scncarbon(t,r) - baucarbon(t,r)) / abatement;
	  );
	  leakage(t,"total") = sum(r, leakage(t,r));
	);

	option leakage:0;
	display leakage;

	deltagdp(t,r)  = scngdp(t,r) - baugdp(t,r);
	pctgdp(t,r)  = 100*(scngdp(t,r) / baugdp(t,r) - 1);
	option pctgdp:1, deltagdp:1;
	display deltagdp, pctgdp;

	pctprc(whe,t,r)$bauprc(whe,t,r) = 
		100*(scnprc(whe,t,r)/bauprc(whe,t,r)-1);
	display pctprc;

*	Carbon imports:

	carbpct(t,r) = na;
	carbval(t,r) = na;

	loop(ttax(r),
	  carbimp(t,r)$ttax(r) = scncarbon(t,r) - carblim(r,t);
	  carbpct(t,r)$((carbimp(t,r) gt 0) and (baucarbon(t,r) gt carblim(r,t)))
		= 100 * carbimp(t,r) / (baucarbon(t,r) - carblim(r,t));
	  carbval(t,r) = 10 * pcarb.l(r) * (scncarbon(t,r) - carblim(r,t)) 
			/ pc.l("usa");
	  carbdom(t,r)$(baucarbon(t,r) - carblim(r,t)) = 
		100* (baucarbon(t,r) - scncarbon(t,r))/(baucarbon(t,r) - carblim(r,t));
	);

	if (card(ttax), display carbimp, carbpct, carbval, carbdom;);

	harbtg(t,r) = 0.5*10*pcarb.l(r)*(baucarbon(t,r) - scncarbon(t,r));
	dwl(t,r) = harbtg(t,r) + carbval(t,r)$card(ttax);

	display harbtg,dwl,carbval;

	pctcpi(t,r) = 100 * ( scncpi(t,r) / baucpi(t,r) - 1);

	ctaxrate(r) = ctax(t,r);

	prices(r,i)$baupp(t,r,i) = 100 * (scnpp(t,r,i) / baupp(t,r,i) - 1);
	cprices(r,i)$baucp(t,r,i) = 100 * (scncp(t,r,i) / baucp(t,r,i) - 1);
	prices(r,"L") = 100 * (scnwage(t,r)/bauwage(t,r) - 1);
	prices(r,"K") = 100 * (scnreturn(t,r)/baureturn(t,r) - 1);
	prices(r,"R")$baurent(t,r) = 100 * (scnrent(t,r)/baurent(t,r) - 1);

	output(r,i)$bauy(i,t,r) = 100 * (scny(i,t,r) / bauy(i,t,r) - 1);
	export(r,i)$sum(rr, bauimport(i,t,r,rr)) = 100 * 
	  (sum(rr, scnimport(i,t,r,rr)) / sum(rr, bauimport(i,t,r,rr)) - 1);
	import(r,i)$sum(rr, bauimport(i,t,rr,r)) = 100 * 
	  (sum(rr, scnimport(i,t,rr,r)) / sum(rr, bauimport(i,t,rr,r)) - 1);
	energytrd(r,wmkt) = scntrade(t,r,wmkt) - bautrade(t,r,wmkt);

	totleakage = leakage(t,"total");
	regleakage(r) = leakage(t,r);

 	verpct(r) = 100 * ( sum((i,s)$ver(i,r,s), pq.l(i,r,s)*ver(i,r,s)) 
			  - sum((i,s)$ver(i,s,r), pq.l(i,s,r)*ver(i,s,r)) ) 
			  / (pc.l(r)*ct0(r)*c.l(r)+sum(cgd,py.l(cgd,r)*id(r))) ; 
);

option verpct:1;
$if %policy%==ver display verpct;

parameter	emissions	Global emissions
		carbtrade	Value of carbon exports (billions);

carbtrade(r)$ttax(r) = 10 * (carbonx.l(r)-carbonm.l(r))*pwcarb.l;

emissions = sum(r, emit.l(r));

$libinclude cgi output totleakage
$libinclude cgi output cpct r
$libinclude cgi output ur r

$libinclude cgi output regleakage r
$libinclude cgi output emissions
$libinclude cgi output gdppct r
$libinclude cgi output ctaxrate r
$libinclude cgi output ctaxpct r fe

set fi(*); fi(i) = yes; fi("L") = yes; fi("K") = yes;

$libinclude cgi output prices r fi
$libinclude cgi output cprices r i
$libinclude cgi output output r i

display cpct, gdppct, totleakage, ctaxrate, ctaxpct, regleakage, import, export, output, prices, energytrd;

if (%border%, display tau.l;);

